﻿/*
===============================================================================
 Name        : cam.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2014.01.01
 Copyright   : Copyright (C) 2014 Ichiro Shirasaka
 Description : MARY_CAM CAMERA DRIVER
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/
#include "cam.h"

#define PARAM_NUM 99

extern volatile int time_count;

/* OV7670 INITALIZE DATA */
const char param[2][PARAM_NUM] = {
		{0x01,0x02,0x03,0x0c,0x0e,0x0f,0x11,0x12,0x15,0x16,0x17,0x18,0x19,0x1a,0x1e,0x21,0x22,
		 0x29,0x32,0x33,0x34,0x35,0x37,0x38,0x39,0x3b,0x3c,0x3d,0x3e,0x3f,0x40,0x41,0x41,
		 0x43,0x44,0x45,0x46,0x47,0x48,0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,
		 0x54,0x56,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,
		 0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x8d,0x8e,0x8f,0x90,
		 0x91,0x96,0x96,0x97,0x98,0x99,0x9a,0x9a,0x9b,0x9c,0x9d,0x9e,0xa2,0xa4,0xb0,
		 0xb1,0xb2,0xb3,0xb8,0xc8,0xc9},
		{0x40,0x60,0x0a,0x00,0x61,0x4b,0xc0,0x04,0x00,0x02,0x13,0x01,0x02,0x7a,0x37,0x02,0x91,
		 0x07,0xb6,0x0b,0x11,0x0b,0x1d,0x71,0x2a,0x12,0x78,0xc3,0x00,0x00,0xd0,0x08,0x38,
		 0x0a,0xf0,0x34,0x58,0x28,0x3a,0x09,0x00,0x40,0x20,0x80,0x80,0x00,0x22,0x5e,
		 0x80,0x40,0x9e,0x88,0x88,0x44,0x67,0x49,0x0e,0x00,0x40,0x0a,0x0a,0x55,0x11,
		 0x9f,0x3a,0x35,0x11,0xf0,0x10,0x05,0xe1,0x01,0x04,0x01,0x4f,0x00,0x00,0x00,
		 0x00,0x00,0x00,0x30,0x20,0x30,0x00,0x84,0x29,0x03,0x4c,0x3f,0x02,0x88,0x84,
		 0x0c,0x0e,0x82,0x0a,0xf0,0x60}
};

/* RESOLUTION DATA 0:address 1:qcif 2:qvga 3:qqvga 4:vga 5:vga-ov7670 crop */
const char sizeparam[6][14] = {
		{0x12,0x17,0x18,0x32,0x19,0x1a,0x03,0x0c,0x3e,0x70,0x71,0x72,0x73,0xa2},
		{0x04,0x39,0x03,0x80,0x03,0x7b,0x02,0x0c,0x11,0x3a,0x35,0x11,0xf1,0x52},
		{0x04,0x16,0x04,0x80,0x02,0x7a,0x0a,0x04,0x19,0x3a,0x35,0x11,0xf1,0x02},
		{0x04,0x16,0x04,0xa4,0x02,0x7a,0x0a,0x04,0x1a,0x3a,0x35,0x22,0xf2,0x02},
		{0x04,0x13,0x01,0xb6,0x02,0x7a,0x0a,0x00,0x00,0x3a,0x35,0x11,0xf0,0x02},
		{0x04,0x33,0x43,0x00,0x2c,0x4c,0x00,0x00,0x00,0x3a,0x35,0x11,0xf0,0x02}};

/* Initialize OV7670 */
void init_cam(void)
{
	int n;

	write_cam(0x12, 0x80);
	delay_ms(100);
	for(n = 0; n < PARAM_NUM; n++){
		write_cam(param[0][n], param[1][n]);		//set camera parameter
		delay_ms(20);
	}
}

/* Change Resolution */
void change_res(int res)
{
	int n;

	for(n = 0; n < 14; n++){
		write_cam(sizeparam[0][n], sizeparam[res][n]);		//set camera parameter
		delay_ms(20);
	}
}


/* write CAM */
void write_cam(unsigned char reg, unsigned char data)
{
    I2CWriteLength = 3;		//I2C write transfer byte = 3
    I2CReadLength = 0;		//I2C read transfer byte = 0
    I2CMasterBuffer[0] = 0x42;		//CAM slave address
    I2CMasterBuffer[1] = reg;	// cam reg address
    I2CMasterBuffer[2] = data;		//data
    I2CEngine();			//Go I2C engine
    return;
}

/* Wait 1ms interrupt */
void delay_count(void)
{
    if(time_count > 0)
    {
	    time_count--;	//count down by 1ms tick interrupt
    }
}

/* Delay ms  */
void delay_ms(int time_ms)
{
    time_count = time_ms;
    while(time_count > 0);	//wait time_count until 0
}
