/*
===============================================================================
 Name        : main.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2014.04.16
 Copyright   : Copyright (C) 2014 Ichiro Shirasaka
 Description : MARY CAMERA MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif


/********************************************************************
*
*  MARY CAMERA Board Test Program
*
*********************************************************************/

#include "cam.h"
#include "spi.h"
#include "gpio.h"
#include "type.h"
#include "i2c.h"
#include "oled.h"
#include "spi.h"

#define WIDE 176
#define HIGHT 144

extern uint8_t cam_slot;	//MAPLE-mini Slot Value=0:Slot1 Value=1:slot2
extern uint8_t oled_slot;	//MAPLE-mini Slot Value=0:Slot1 Value=1:slot2

/* Prototype */
void cam_test(void);

/* Test Program Main routine */
int main(void)
{
	int32_t x, y;			//OLED pixel location
	uint32_t m_address;		//Flame buffer address
	uint16_t buf[128];		//Line buffer
	int res = 0, res_count = 0;		//resolution, resolution change count
	int ress[6][2] = {				//resolution type
			{  0,  0},				//Dummy
			{176,144},				//QCIF(176x144)
			{320,240},				//QVGA(320x240)
			{160,120},				//QQVA(160x120)
			{640,480},				//VGA(640x480)
			{128,128},				//VGA(640x480 Crop with OV7670)
	};

	cam_slot = SLOT2;						//MARY CAMERA SLOT = SLOT2
	oled_slot = SLOT1;						//OLED SLOT = SLOT1
	GPIOInit();										//Initialize GPIO
	if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )	//initialize I2c
	{
 		while ( 1 );								//Fatal error
	}
	SysTick_Config(SystemCoreClock / 1000);		   //SysTick interrupt=1ms
	LPC_GPIO->W0[21] = 0;					//CQ ARM Board LED1 ON
	LPC_GPIO->W1[15] = 0;					//Reset CAM Board
	LPC_GPIO->W1[15] = 1;					//
	Init_OLED();							//Reset OLED Board
	OLED_printf_Font(OLED_FONT_SMALL);		//Set OLED FONT
	OLED_printf_Position(0, 3);				//Set print location
	OLED_printf_Color(OLED_MAG, OLED_BLK);	//Set print Color
	OLED_printf(" MARY CAMERA\n TEST\n");	//print OLED

/* MAIN LOOP */
	while(1){
		/* Change resolution */
		if(res_count == 0){						//Check change resolution timing
			res++;									//Change resolution
			res_count = 127;
			if(res > 5) res = 1;
			OLED_printf_Position(0, 15);
			OLED_printf_Color(OLED_WHT, OLED_BLK);
			OLED_printf("RES=%dX%d",ress[res][0],ress[res][1]);
			write_cam(0x11, 0x80);					//Reset camera module
			delay_ms(100);
			init_cam();								//Initialize CAM logic
			change_res(res);
		}
		res_count--;								//Update resolution counter
		Switch_spi(8, SPI_FAST, SPI_GENERIC);		//Set CAMERA SPI mode
		//LPC_GPIO->W1[19] = 0;						//Set Read  Use READ_RQ = 0
		SPI_TxData(0x03,cam_slot);					//Set frame buffer read request Use SPI
		SPI_TxData(0x01,cam_slot);
		//while(LPC_GPIO->W0[20] == 0 );			//wait data ready  Use READY signal
		SPI_TxData(0x84,cam_slot);
		while((SPI_RxData(0x84,cam_slot) & 0x01) == 0 );	//wait data ready Use SPI
		LPC_GPIO->W0[21] = 0;						//ARM Board LED2 ON
		//Draw odd line
		m_address = ((ress[res][0] - 128) / 2)*2+ress[res][0]*((ress[res][1] - 128) / 2)*2;//Calculate Line memory address
		for(y = 0; y < 128; y += 2){				//1 fame loop
			Switch_spi(8, SPI_FAST, SPI_GENERIC);	//Set CAMERA SPI mode
			SPI_TxData(0x00,cam_slot);				//set line memory address low byte
			SPI_TxData(m_address & 0xff,cam_slot);
			SPI_TxData(0x01,cam_slot);				//set line memory address middle byte
			SPI_TxData((m_address >> 8) & 0xff,cam_slot);
			SPI_TxData(0x02,cam_slot);				//set line memory address high byte
			SPI_TxData((m_address >> 16) & 0xff,cam_slot);
			m_address += ress[res][0]*4;			//update memory address
			for(x = 0; x < 128; x++){				//1 line loop
					buf[x] = (SPI_RxData(0xa8,cam_slot) << 8) | SPI_RxData(0xa8,cam_slot);	//read 1 line data from VCAMB
			}
			OLED_Put_Line(y, buf);					//Draw 1 line OLED
		}
		//Draw even line
		m_address = ((ress[res][0] - 128) / 2)*2+ress[res][0]*((ress[res][1] - 128) / 2)*2;
		m_address += ress[res][0]*2;
		for(y = 1; y < 128; y += 2){				//1 fame loop
			Switch_spi(8, SPI_FAST, SPI_GENERIC);	//Set CAMERA SPI mode
			SPI_TxData(0x00,cam_slot);				//set line memory address low byte
			SPI_TxData(m_address & 0xff,cam_slot);
			SPI_TxData(0x01,cam_slot);				//set line memory address middle byte
			SPI_TxData((m_address >> 8) & 0xff,cam_slot);
			SPI_TxData(0x02,cam_slot);				//set line memory address high byte
			SPI_TxData((m_address >> 16) & 0xff,cam_slot);
			m_address +=ress[res][0]*4;				//update memory address
			for(x = 0; x < 128; x++){				//1 line loop
					buf[x] = (SPI_RxData(0xa8,cam_slot) << 8) | SPI_RxData(0xa8,cam_slot);	//read 1 line data from VCAMB
			}
			OLED_Put_Line(y, buf);					//Draw 1 line OLED
		}
		Switch_spi(8, SPI_FAST, SPI_GENERIC);	//Set CAMERA SPI mode
		//LPC_GPIO->W1[19] = 1;					//Set Read  Use READ_RQ = 1
		SPI_TxData(0x03,cam_slot);				//Set frame buffer read request Use SPI
		SPI_TxData(0x00,cam_slot);
		//while(LPC_GPIO->W0[20] != 0 );		//wait data ready  Use READY signal
		SPI_TxData(0x84,cam_slot);
		while((SPI_RxData(0x84,cam_slot) & 0x01) != 0 );	//wait data ready Use SPI
		LPC_GPIO->W0[21] = 1;					//ARM Board LED2 OFF
	}
    return 0 ;
}

/* GPIO initialize */
void GPIOInit(void)
{
/* Initialize GPIO */
/*
 * PIO0_0:SW1(RESET)
 *
 * PIO0_4:SCL
 * PIO0_5:SDA
 *
 * PIO0_6:USB PULLUP ENABLE
 *
 * PIO0_20:Debuger LED1
 * PIO0_21:Debuger LED2
 *
 * PIO0_9:MOSI0(SPI)
 * PIO0_8:MISO0(SPI)
 * PIO0_10:SCK0(SPI)
 * PIO0_2:VCAMB_CS(SPI)
 * PIO0_16:OLED_CS(SPI)
 *
 *
 */
	LPC_SYSCON->SYSAHBCLKCTRL |= ((1<<19) | (1<<23) | (1<<24));
	LPC_GPIO->DIR[0] = 0b001000010011100000000000;	//Refer up
	LPC_GPIO->DIR[1] = 0b000010001000000000000000;	//Refer up
	LPC_GPIO->W0[21] = 1;	//ARM Board LED2:OFF
	LPC_GPIO->W1[15] = 1;	//RESET:OFF
	LPC_GPIO->W1[19] = 1;	//READ_RQ:OFF
}
